#!/usr/bin/env python
# PControl.py
# created by SAB (2006?)
# based on Matlab PControl by MBR (2004?)
# modified by SAB, AMW
# current version JAB 4/8/07

import os
import sys
import wx
from wx import xrc

import motmot.wxvalidatedtext.wxvalidatedtext as wxvt

import Panel_com
import listener

import pkg_resources # part of setuptools
RESFILE = pkg_resources.resource_filename(__name__,"panel_control.xrc")

button_start_color = "#00EE00"
button_stop_color = "#EE0000"

class PControl_App( wx.App ):
    def OnInit(self):
        """Initialization: bind objects and events."""
        try: # see if port name was sent on command line
            userport = sys.argv[1]
        except IndexError:
            userport = '/dev/ttyS0'
        # create serial communication object
        self.comm = Panel_com.Panel_com( userport )

        # create empty interface objects
        self.Flytrax_listener = None
        self.Spheretrax_listener = None

        # make frame from .xrc file
        rsrc = xrc.EmptyXmlResource()
        rsrc.LoadFromString(open(RESFILE).read())

        self.frame = rsrc.LoadFrame(None, "frame_panel_control") # make frame main panel

        # make handles to useful objects
        self.menu = self.frame.GetMenuBar()
        self.start_button = xrc.XRCCTRL( self.frame, "button_start" )
        self.start_button.SetBackgroundColour( button_start_color )
        self.pattern_id_spinner = xrc.XRCCTRL( self.frame, "spin_pattern" )
        self.pattern_x_spinner = xrc.XRCCTRL( self.frame, "spin_pos_x" )
        self.pattern_y_spinner = xrc.XRCCTRL( self.frame, "spin_pos_y" )
        self.pos_x_text = xrc.XRCCTRL( self.frame, "text_n_pos_x" )
        self.pos_y_text = xrc.XRCCTRL( self.frame, "text_n_pos_y" )
        self.gain_x_box = xrc.XRCCTRL( self.frame, "GAIN_X" )
        self.offset_x_box = xrc.XRCCTRL( self.frame, "OFFSET_X" )
        self.gain_y_box = xrc.XRCCTRL( self.frame, "GAIN_Y" )
        self.offset_y_box = xrc.XRCCTRL( self.frame, "OFFSET_Y" )

        # set up text validators
        wxvt.setup_validated_integer_callback( self.pos_x_text,
                                               xrc.XRCID("text_n_pos_x"),
                                               self.OnPosTextValid )
        wxvt.setup_validated_integer_callback( self.pos_y_text,
                                               xrc.XRCID("text_n_pos_y"),
                                               self.OnPosTextValid )

        # bind callbacks
        # menu callbacks
        self.frame.Bind( wx.EVT_MENU, self.OnReset, id=xrc.XRCID("menu_command_reset") )
        self.frame.Bind( wx.EVT_MENU, self.OnBlink, id=xrc.XRCID("menu_command_blink") )
        self.frame.Bind( wx.EVT_MENU, self.OnAllOn, id=xrc.XRCID("menu_command_all_on") )
        self.frame.Bind( wx.EVT_MENU, self.OnAllOff, id=xrc.XRCID("menu_command_all_off") )
        self.frame.Bind( wx.EVT_MENU, self.OnGreenscale, id=xrc.XRCID("menu_greenscale_0") )
        self.frame.Bind( wx.EVT_MENU, self.OnGreenscale, id=xrc.XRCID("menu_greenscale_1") )
        self.frame.Bind( wx.EVT_MENU, self.OnGreenscale, id=xrc.XRCID("menu_greenscale_2") )
        self.frame.Bind( wx.EVT_MENU, self.OnGreenscale, id=xrc.XRCID("menu_greenscale_3") )
        self.frame.Bind( wx.EVT_MENU, self.OnGreenscale, id=xrc.XRCID("menu_greenscale_4") )
        self.frame.Bind( wx.EVT_MENU, self.OnGreenscale, id=xrc.XRCID("menu_greenscale_5") )
        self.frame.Bind( wx.EVT_MENU, self.OnGreenscale, id=xrc.XRCID("menu_greenscale_7") )
        self.frame.Bind( wx.EVT_MENU, self.OnPanelAddress, id=xrc.XRCID("menu_command_address") )
        self.frame.Bind( wx.EVT_MENU, self.OnInterface, id=xrc.XRCID("menu_interface_flytrax") )
        self.frame.Bind( wx.EVT_MENU, self.OnInterface, id=xrc.XRCID("menu_interface_spheretrax") )
        self.frame.Bind( wx.EVT_MENU, self.OnQuit, id=xrc.XRCID("menu_quit_quit") )
        # button callbacks
        self.frame.Bind( wx.EVT_BUTTON, self.OnSetPatternID, id=xrc.XRCID("button_set_pattern") )
        self.frame.Bind( wx.EVT_BUTTON, self.OnSetGainOffset, id=xrc.XRCID("button_set_gains") )
        self.frame.Bind( wx.EVT_BUTTON, self.OnStart, id=xrc.XRCID("button_start") )
        # spinner callbacks
        self.frame.Bind( wx.EVT_SPINCTRL, self.OnPositionSpinner, id=xrc.XRCID("spin_pos_x") )
        self.frame.Bind( wx.EVT_SPINCTRL, self.OnPositionSpinner, id=xrc.XRCID("spin_pos_y") )
        # miscellaneous callbacks
        self.frame.Bind( wx.EVT_CLOSE, self.OnQuit )

        self.frame.Show()
        return True

    def OnReset( self, evt ):
        self.comm.Reset()

    def OnBlink( self, evt ):
        self.comm.BlinkLED()

    def OnAllOn( self, evt ):
        self.comm.AllOn()

    def OnAllOff( self, evt ):
        self.comm.AllOff()

    def OnGreenscale( self, evt ):
        if evt.GetId() == xrc.XRCID( "menu_greenscale_0" ): level = 0
        elif evt.GetId() == xrc.XRCID( "menu_greenscale_1" ): level = 1
        elif evt.GetId() == xrc.XRCID( "menu_greenscale_2" ): level = 2
        elif evt.GetId() == xrc.XRCID( "menu_greenscale_3" ): level = 3
        elif evt.GetId() == xrc.XRCID( "menu_greenscale_4" ): level = 4
        elif evt.GetId() == xrc.XRCID( "menu_greenscale_5" ): level = 5
        elif evt.GetId() == xrc.XRCID( "menu_greenscale_6" ): level = 6
        elif evt.GetId() == xrc.XRCID( "menu_greenscale_7" ): level = 7
        else:
            wx.MessageBox( "can't set that level --\nGUI is messed up", "Error", wx.ICON_ERROR )
        self.comm.Greenscale( level )

    def OnPanelAddress( self, evt ):
        import warnings
        warnings.filterwarnings( 'ignore', '', DeprecationWarning )
        # function deprecated, to be replaced with wx.NumberEntryDialog,
        # though wx.NumberEntryDialog doesn't exist yet
        oldaddr = wx.GetNumberFromUser( "Enter old panel address", "(0-255)", "Old address", 0, min=0, max=255, parent=self.frame )
        if oldaddr < 0: return
        newaddr = wx.GetNumberFromUser( "Enter new panel address", "(0-255)", "New address", 0, min=0, max=255, parent=self.frame )
        if newaddr < 0: return
        warnings.resetwarnings()
        self.comm.Address( oldaddr, newaddr )

    def OnInterface( self, evt ):
        # determine which interface we're dealing with
        if evt.GetId() == xrc.XRCID( "menu_interface_flytrax" ):
            f_callback = True
            s_callback = False
            this_id = xrc.XRCID( "menu_interface_flytrax" )
        elif evt.GetId() == xrc.XRCID( "menu_interface_spheretrax" ):
            f_callback = False
            s_callback = True
            this_id = xrc.XRCID( "menu_interface_spheretrax" )
        else:
            wx.MessageBox( "unknown interface -- GUI error", "Error", wx.ICON_ERROR )
            return

        # check to make sure that both interfaces are not running simultaneously
        # TODO: do we really care?
        if self.menu.IsChecked( xrc.XRCID("menu_interface_flytrax") ) and \
           self.menu.IsChecked( xrc.XRCID("menu_interface_spheretrax") ):
            wx.MessageBox( "cannot run multiple interfaces", "Error", wx.ICON_ERROR )
            # uncheck the one that this callback was trying to check...
            self.menu.Check( this_id, False )
            return

        if f_callback:
            if self.menu.IsChecked( this_id ):
                self.Flytrax_listener = listener.Flytrax_listener( self.comm )
                if not self.Flytrax_listener.init_successful:
                    self.Flytrax_listener = None
                    self.menu.Check( this_id, False )
                    wx.MessageBox( "Flytrax listener cancelled", "Cancelled", wx.ICON_INFORMATION )
                    return
            else:
                self.Flytrax_listener = None
        elif s_callback:
            if self.menu.IsChecked( this_id ):
                self.Spheretrax_listener = listener.Spheretrax_listener( self.comm )
                if not self.Spheretrax_listener.init_successful:
                    self.Spheretrax_listener = None
                    self.menu.Check( this_id, False )
                    wx.MessageBox( "Sheretrax listener cancelled", "Cancelled", wx.ICON_INFORMATION )
                    return
            else:
                self.Spheretrax_listener = None

    def OnQuit( self, evt ):
        """Close window. Put any other clean-up code here."""
        if self.Flytrax_listener is not None:
            self.Flytrax_listener = None # garbage collection will now stop thread
        if self.Spheretrax_listener is not None:
            self.Spheretrax_listener = None
        self.frame.Destroy()

    def OnSetPatternID(self,event):
        """Set pattern ID given current spinner value."""
	patternid = self.pattern_id_spinner.GetValue()
        try: self.comm.SetPatternID(patternid)
        except ValueError:
            wx.MessageBox( "pattern ID out of range", "Error", wx.ICON_ERROR )

    def OnPosTextValid( self, evt ):
        """Set number of pattern positions given text-box values."""
        # get values based on event ID
        if evt.GetId() == xrc.XRCID( "text_n_pos_x" ):
            new_max = int(self.pos_x_text.GetValue())-1
            spinner = self.pattern_x_spinner
        elif evt.GetId() == xrc.XRCID( "text_n_pos_y" ):
            new_max = int(self.pos_y_text.GetValue())-1
            spinner = self.pattern_y_spinner

        # generic settings changes
        spinner.SetRange( 0, new_max )
        if spinner.GetValue() > new_max:
            spinner.SetValue( new_max )

    def OnSetGainOffset(self,event):
	gainx = int(self.gain_x_box.GetValue())
	offsetx = int(self.offset_x_box.GetValue())
	gainy = int(self.gain_y_box.GetValue())
	offsety = int(self.offset_y_box.GetValue())
        try: self.comm.SetGainOffset(gainx,offsetx,gainy,offsety)
        except ValueError:
            wx.MessageBox( "one of the gain or offset values\nwas out of the allowed range", "Error", wx.ICON_ERROR )

    def OnStart(self,event):
        if self.start_button.GetLabel() == "Start":
            self.start_button.SetLabel( "Stop" )
            self.start_button.SetBackgroundColour( button_stop_color )
            if self.Flytrax_listener is not None:
                pass
            elif self.Spheretrax_listener is not None:
                pass
            else:
                self.comm.Start()
        elif self.start_button.GetLabel() == "Stop":
            self.start_button.SetLabel( "Start" )
            self.start_button.SetBackgroundColour( button_start_color )
            if self.Flytrax_listener is not None:
                pass
            elif self.Spheretrax_listener is not None:
                pass
            else:
                self.comm.Stop()
        else:
            wx.MessageBox( "can't start or stop --\nGUI is messed up", "Error", wx.ICON_ERROR )

    def OnPositionSpinner(self,event):
	xpos = self.pattern_x_spinner.GetValue()
	ypos = self.pattern_y_spinner.GetValue()
        try: self.comm.SetPositions(xpos,ypos)
        except ValueError:
            wx.MessageBox( "one of the x or y position values\nwas out of the allowed range", "Error", wx.ICON_ERROR )

def main():
    app = PControl_App( 0 )
    app.MainLoop()

if __name__ == '__main__':
    main()
