# Panel_com.py
# original in Matlab by MBR (2004?)
# translated into Python by SAB (2006?)
# made into a class by JAB 4/8/07

import serial
import types

class Panel_com:
    def __init__( self, userport='/dev/ttyS0' ):
        self.ser = serial.Serial(
            port=userport,               #number of device, numbering starts at
                                         #zero. if everything fails, the user
                                         #can specify a device string, note
                                         #that this isn't portable anymore
                                         #if no port is specified an unconfigured
                                         #an closed serial port object is created
            baudrate=19200,              #baudrate
            bytesize=serial.EIGHTBITS,   #number of databits
            parity=serial.PARITY_NONE,   #enable parity checking
            stopbits=serial.STOPBITS_ONE,#number of stopbits
            timeout=None,                #set a timeout value, None for waiting forever
            xonxoff=0,                   #enable software flow control
            rtscts=0,                    #enable RTS/CTS flow control
        )

    #################################################################################
    def SetPatternID(self,patternid):
        if type(patternid) != types.IntType: raise TypeError
        if patternid < 0: raise ValueError
        self.send(chr(0x02) + chr(0x03) + chr(patternid))

    def BlinkLED( self ): self.send(chr(0x01) + chr(0x50))

    def AllOn( self ): self.send(chr(0x01) + chr(0xFF))

    def AllOff( self ): self.send(chr(0x01) + chr(0x00))

    def Greenscale( self, level ):
        if type(level) != types.IntType: raise TypeError
        if level < 0 or level > 7: raise ValueError
        self.send(chr(0x01) + chr(0x40 + level))

    def Start( self ): self.send(chr(0x01) + chr(0x20))

    def Stop( self ): self.send(chr(0x01) + chr(0x30))

    def Reset( self ): self.send(chr(0x02) + chr(0x01) + chr(0x00))

    def SetMode( self, modex, modey ):
        if type(modex) != types.IntType or type(modey) != types.IntType:
            raise TypeError
        if modex < 0 or modey < 0 or modex > 4 or modey > 4:
            raise ValueError
        self.send(chr(0x03) + chr(0x10) + chr(modex) + chr(modey))

    def SetGainOffset(self,gainx,offsetx,gainy,offsety):
        if type(gainx) != types.IntType or type(offsetx) != types.IntType or \
           type(gainy) != types.IntType or type(offsety) != types.IntType:
            raise TypeError
        self.send(chr(0x05) + chr(0x71) + chr(gainx) + chr(offsetx) + chr(gainy) + chr(offsety))

    def SetPositions(self,xpos,ypos):
        if type(xpos) != types.IntType or type(ypos) != types.IntType:
            raise TypeError
        if xpos < 0 or xpos > 255 or ypos < 0 or ypos > 255:
            raise ValueError
        self.send(chr(0x05) + chr(0x70) + chr(xpos) + chr(0x00) + chr(ypos) + chr(0x00))

    def Address( self, oldaddr, newaddr ):
        if type(oldaddr) != types.IntType or type(newaddr) != types.IntType:
            raise TypeError
        if oldaddr < 0 or newaddr < 0: raise ValueError
        self.send(chr(0x03) + chr(0xFF) + chr(oldaddr) + chr(newaddr))

    #################################################################################
    def send(self,buf):
        #print "sending:", repr(buf)
        self.ser.write( buf )
